import {
  NodeTypes,
  TRACE_ID,
  TagTypes,
  __spreadProps,
  __spreadValues
} from "./chunk-JIXMXCRK.mjs";

// src/core/index.ts
import { createUnplugin } from "unplugin";

// src/core/parse_ID.ts
import { extname } from "path";
function parse_ID(id, root = "") {
  const [file, rawQuery] = id.split("?", 2);
  const ext = extname(file).slice(1);
  const isSfc = ext === "vue";
  const isTsx = ext === "tsx";
  const isJsx = isTsx || ext === "jsx";
  const query = Object.fromEntries(new URLSearchParams(rawQuery));
  if (query.vue != null) {
    query.vue = true;
  }
  return {
    file: file.replace(root, ""),
    isSfc,
    isTsx,
    isJsx,
    query
  };
}

// src/core/filter_ID.ts
function filter_ID(id) {
  const parsed = parse_ID(id);
  if (parsed.isJsx) {
    return true;
  }
  if (parsed.isSfc) {
    const { query } = parsed;
    if (query.type === "template") {
      return true;
    }
    return (
      // vite-plugin-vue
      !query[TRACE_ID] && // rollup-plugin-vue
      !query["rollup-plugin-vue"]
    );
  }
  return false;
}

// src/core/transform.ts
import MagicString from "magic-string";

// src/core/transform_SFC.ts
import { parse as parse2, transform } from "@vue/compiler-dom";

// src/core/transform_JSX.ts
import { traverse, types as t } from "@babel/core";
import { parse } from "@babel/parser";
function transform_JSX(code, cb, options) {
  const { isTsx, startIndex = 0, startLine = 1, startColumn = 1 } = options;
  const plugins = ["jsx"];
  if (isTsx) {
    plugins.push("typescript");
  }
  const ast = parse(code, {
    sourceType: "unambiguous",
    plugins,
    startLine,
    // babel start at 0
    startColumn: startColumn - 1
  });
  traverse(ast, {
    JSXOpeningElement({ node }) {
      const nameNode = node.name;
      if (!nameNode)
        return;
      const { start } = node.loc;
      const name = getJSXElementName(nameNode);
      const offset = start.index + startIndex + name.length + 1;
      cb(__spreadProps(__spreadValues({}, start), {
        // babel starts at 0, so we need to add 1
        column: start.column + 1,
        offset
      }));
    }
  });
}
function getJSXElementName(nameNode) {
  let nameValue;
  if (t.isJSXIdentifier(nameNode)) {
    nameValue = nameNode.name;
  } else if (t.isJSXNamespacedName(nameNode)) {
    const { namespace, name } = nameNode;
    nameValue = `${namespace.name}:${name.name}`;
  } else {
    const nameValues = [];
    while (t.isJSXMemberExpression(nameNode)) {
      nameValues.unshift(nameNode.property.name);
      nameNode = nameNode.object;
    }
    nameValues.unshift(nameNode.name);
    nameValue = nameValues.join(".");
  }
  return nameValue;
}

// src/core/transform_SFC.ts
function transform_SFC(code, cb) {
  const ast = parse2(code);
  transform(ast, {
    nodeTransforms: [
      (node) => {
        if (node.type === NodeTypes.ELEMENT && TagTypes.includes(node.tagType)) {
          const { start } = node.loc;
          const offset = start.offset + node.tag.length + 1;
          cb(__spreadProps(__spreadValues({}, start), {
            offset
          }));
        }
      }
    ]
  });
  const jsxOpts = resolveJsxOptions(ast);
  if (jsxOpts) {
    transform_JSX(jsxOpts.code, cb, jsxOpts);
  }
}
function resolveJsxOptions(ast) {
  var _a;
  const scriptNode = ast.children.find(
    (node) => node.tag === "script"
  );
  if (!scriptNode)
    return;
  const codeNode = scriptNode.children[0];
  if (!codeNode)
    return;
  const langProp = scriptNode.props.find(
    (prop) => prop.name === "lang"
  );
  if (!langProp)
    return;
  const lang = (_a = langProp.value) == null ? void 0 : _a.content;
  const isTsx = lang === "tsx";
  const isJsx = isTsx || lang === "jsx";
  if (isJsx) {
    const { offset, line, column } = codeNode.loc.start;
    return {
      isTsx,
      startIndex: offset,
      startLine: line,
      startColumn: column,
      code: codeNode.content
    };
  }
}

// src/core/transform.ts
function transform2(code, id, options) {
  const { root, sourceMap } = options;
  const s = new MagicString(code);
  const parsed = parse_ID(id, root);
  if (parsed.isSfc) {
    transform_SFC(code, replace);
  } else if (parsed.isJsx) {
    transform_JSX(code, replace, parsed);
  }
  function replace(pos) {
    const { offset, line, column } = pos;
    s.prependLeft(offset, ` ${TRACE_ID}="${parsed.file}:${line}:${column}"`);
  }
  return {
    code: s.toString(),
    map: sourceMap ? s.generateMap() : null
  };
}

// src/core/index.ts
var unpluginFactory = (options = {}) => {
  if (process.env.NODE_ENV !== "development") {
    return {
      name: "unplugin-vue-source"
    };
  }
  const opts = resolveOptions(options);
  return {
    name: "unplugin-vue-source",
    enforce: "pre",
    transformInclude: filter_ID,
    transform(code, id) {
      return transform2(code, id, opts);
    }
  };
};
function resolveOptions(options) {
  const { root = process.cwd(), sourceMap = false } = options;
  return {
    root,
    sourceMap
  };
}
var core_default = /* @__PURE__ */ createUnplugin(unpluginFactory);

export {
  core_default
};
